struct sLeaderBoard {
  int score  = 0;
  char Init1 = ' ';
  char Init2 = ' ';
};

// A typedef is convenient when you need to create multiple versions/copies of a same datatype 
//   it is the same as int or long or char
typedef struct sLeaderBoard sLeaderBoard_t;

const int numLeaders = 3;   // many ways of doing this but this is a simple 3 entry leaderboard to test concepts

sLeaderBoard_t OldLeaderBoard[numLeaders];
sLeaderBoard_t NewLeaderBoard[numLeaders];
sLeaderBoard_t CurrentPlayer;

void setup() {
  
  Serial.begin(115200);
  while (Serial.available()) Serial.read(); // flush Serial so wait for any key will wait
  while (!Serial.availableForWrite()) {}    // Wait until Serial port is ready for output
}

void loop() {

  WaitForAnyKey("Press any key to continue");

  CreateTestPlayer(12000, 'Y', '2' );
  DetermineNewLeaders();              // now first entry will end up 2nd
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();

  CreateTestPlayer( 8000, 'Z', '3' );
  DetermineNewLeaders();              // now second entry will end up 3rd
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();

  CreateTestPlayer(17000, 'X', '1' );
  DetermineNewLeaders();              // now first entry
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();

  delay(5000);
  Serial.printf("\n\nStart test1 Current too small for top 3");
  CreateTestPlayer(7000, 'A', 'a');   // current player = 7000 (less than lowest)
  DetermineNewLeaders();    //  s/b old = 17000, 12000, 8000 // then new = 17000, 12000, 8000
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();
  delay(5000);

  Serial.printf("\n\nStart test2 Current goes between 12000 and 8000");
  CreateTestPlayer(9000, 'B', 'b');   // current player = 9000 should replace 8000
  DetermineNewLeaders();    //  s/b old = 17000, 12000, 8000  // then new = 17000, 12000, 9000
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();
  delay(5000);

  Serial.printf("\n\nStart test3 Current goes between 17000 and 12000");
  CreateTestPlayer(13000, 'C', 'c');    // current player = 13000 should push 9000 off and go between 12000 and 17000
  DetermineNewLeaders();    //  s/b old = 17000, 12000, 9000  // then new = 17000, 13000, 12000
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();
  delay(5000);

  Serial.printf("\n\nStart test4 Current is highest score");
  CreateTestPlayer(18000, 'D', 'd');    // current player = 18000 should push 12000 off and is new highest at 18000
  DetermineNewLeaders();    //  s/b old = 17000, 13000, 12000 // then new = 18000, 17000, 13000
  Printall();
  CopyNewLeaderBoardToOldLeaderBoard();
  delay(5000);
}

void WaitForAnyKey(String Msg){

  Serial.println(" ");
  Serial.println(Msg);
  while(!Serial.available()) {};  // wait for a key
  Serial.read();                  // read the key and throw away
}

void CreateTestPlayer(int Score, char Init1, char Init2){

  CurrentPlayer.score = Score;
  CurrentPlayer.Init1 = Init1;
  CurrentPlayer.Init2 = Init2 ;

  Serial.printf("\n\nAdding %c%c %i\n", Init1, Init2, Score);
}

void CopyNewLeaderBoardToOldLeaderBoard(){

  for (int indxLeaders = 0; indxLeaders < numLeaders; ++indxLeaders){
    OldLeaderBoard[indxLeaders] = NewLeaderBoard[indxLeaders];  // Allow compiler do job of copying all fields of the structure
  }
}

void DetermineNewLeaders(){
  int indxOld = 0;  // loop over OldLeaderBoard with manual control in order to insert a higher score
  int indxNew = 0;  // loop over NewLeaderBoard in for loop as this is the target and must be numLeaders in size

  for (int indxNew = 0; indxNew < numLeaders; indxNew++){
    if (OldLeaderBoard[indxOld].score > CurrentPlayer.score){   // Is Old greater than Current
      NewLeaderBoard[indxNew] = OldLeaderBoard[indxOld];        // YES it is so copy Old to New
      indxOld++;                                                // Advance the Old index
    }
    else {                                                      // NO it is not
      NewLeaderBoard[indxNew] = CurrentPlayer;                  // so copy Current to New
      CurrentPlayer = {};                                       // Make Current smallest so it is not copied again
    }
  }
}

void Printall(){

  Serial.printf("\nOld Leader Board");
  for (int indxLeaders = 0; indxLeaders < numLeaders; indxLeaders++){
    Serial.printf("\nInitials=%c%c score=%i", 
      OldLeaderBoard[indxLeaders].Init1, OldLeaderBoard[indxLeaders].Init2, OldLeaderBoard[indxLeaders].score);
  }

  Serial.printf("\n\nNew Leader Board");
  for (int indxLeaders = 0; indxLeaders < numLeaders; indxLeaders++){
    Serial.printf("\nInitials=%c%c score=%i", 
      NewLeaderBoard[indxLeaders].Init1, NewLeaderBoard[indxLeaders].Init2, NewLeaderBoard[indxLeaders].score);
  }
}
